﻿; SQLite3u.ahk
; Web Address for SQL commands :


;===============================================================================
; SQLite Globals
;===============================================================================

; Default path of SQLite3.dll (A_WorkingDir)
; Required in _SQLite_Startup()
$SQLITE_s_DLL := "sqlite3.dll"

; SQLite3.dll handle
; Returned from _SQLite_Startup()
; Required in _SQLite_Shutdown()
$SQLITE_h_DLL := 0

; Handle of last opened Database;
; Returned from _SQLite_OpenDB()
$SQLITE_h_DB := 0

; Handle of last prepared Query
; Returned from _SQLite_Query()
$SQLITE_h_QUERY := 0

; Functions errormessages
$SQLITE_s_ERROR := ""

;===============================================================================
; SQL As Understood By SQLite
;   http://www.sqlite.org/lang.html
;===============================================================================
/*
aggregate functions        CREATE VIRTUAL TABLE           INSERT              
ALTER TABLE                date and time functions        keywords             
ANALYZE                    DELETE                         ON CONFLICT clause   
ATTACH DATABASE            DETACH DATABASE                PRAGMA               
BEGIN TRANSACTION          DROP INDEX                     REINDEX              
comment                    DROP TABLE                     RELEASE SAVEPOINT    
COMMIT TRANSACTION         DROP TRIGGER                   REPLACE              
core functions             DROP VIEW                      ROLLBACK TRANSACTION 
CREATE INDEX               END TRANSACTION                SAVEPOINT            
CREATE TABLE               EXPLAIN                        SELECT               
CREATE TRIGGER             expression                     UPDATE               
CREATE VIEW                INDEXED BY                     VACUUM               
*/

;===============================================================================
; Functions:
;   http://www.sqlite.org/c3ref/funclist.html
;===============================================================================
/*
_SQLite_Startup()                                       ; Loads SQLite3.dll
_SQLite_Shutdown()                                      ; Unloads SQLite3.dll
_SQLite_OpenDB($sDBFile)                                ; Opens Database, Sets Standard Handle, Returns Handle
_SQLite_CloseDB($hDB | -1)                              ; Closes Database

_SQLite_Prepare_V2($hDB | -1 , $sSQL , ByRef $hQuery)   ; Prepares $sSql, Returns Error Code
_SQLite_Finalize($hQuery)                               ; Free prepared query

_SQLite_Exec($hDB | -1 , $sSQL)                         ; Executes $sSQL (No Result), Returns Error Code
;;; Prepare_V2, Step, Finalize

_SQLite_GetTable($hDB | -1 , $sSQL , ByRef $sResult , ByRef $iRows , ByRef $iCols ,
                [$iMaxResult = -1], [$iCharSize = 64])  ; Executes $sSQL Query to $sResult, Returns Error Code
;;; Prepare_V2, FetchNames, FetchData, Finalize
*/

;===============================================================================
; Result Codes
;   http://www.sqlite.org/c3ref/c_abort.html
;===============================================================================
$SQLITE_OK           :=  0    ; Successful result 
$SQLITE_ERROR        :=  1    ; SQL error or missing database 
$SQLITE_INTERNAL     :=  2    ; Internal logic error in SQLite 
$SQLITE_PERM         :=  3    ; Access permission denied 
$SQLITE_ABORT        :=  4    ; Callback routine requested an abort 
$SQLITE_BUSY         :=  5    ; The database file is locked 
$SQLITE_LOCKED       :=  6    ; A table in the database is locked 
$SQLITE_NOMEM        :=  7    ; A malloc() failed 
$SQLITE_READONLY     :=  8    ; Attempt to write a readonly database 
$SQLITE_INTERRUPT    :=  9    ; Operation terminated by sqlite3_interrupt()
$SQLITE_IOERR        := 10    ; Some kind of disk I/O error occurred 
$SQLITE_CORRUPT      := 11    ; The database disk image is malformed 
$SQLITE_NOTFOUND     := 12    ; NOT USED. Table or record not found 
$SQLITE_FULL         := 13    ; Insertion failed because database is full 
$SQLITE_CANTOPEN     := 14    ; Unable to open the database file 
$SQLITE_PROTOCOL     := 15    ; Database lock protocol error 
$SQLITE_EMPTY        := 16    ; Database is empty 
$SQLITE_SCHEMA       := 17    ; The database schema changed 
$SQLITE_TOOBIG       := 18    ; String or BLOB exceeds size limit 
$SQLITE_CONSTRAINT   := 19    ; Abort due to constraint violation 
$SQLITE_MISMATCH     := 20    ; Data type mismatch 
$SQLITE_MISUSE       := 21    ; Library used incorrectly 
$SQLITE_NOLFS        := 22    ; Uses OS features not supported on host 
$SQLITE_AUTH         := 23    ; Authorization denied 
$SQLITE_FORMAT       := 24    ; Auxiliary database format error 
$SQLITE_RANGE        := 25    ; 2nd parameter to sqlite3_bind out of range 
$SQLITE_NOTADB       := 26    ; File opened that is not a database file 
$SQLITE_ROW          := 100   ; sqlite3_step() has another row ready 
$SQLITE_DONE         := 101   ; sqlite3_step() has finished executing 

;===============================================================================
; Function Name:    _SQLite_StartUP()
; Description:      Load SQLite3.dll
; Requirement(s):   Valid Path to SQlite3.dll in $SQLITE_s_DLL
; Parameter(s):     None
; Return Value(s):  On Success - ErrorLevel = 0 = $SQLITE_OK
;                   On Failure - ErrorLevel
;===============================================================================
_SQLite_Startup()
{
  Global
  $SQLITE_h_DLL := DllCall("LoadLibrary", "Str", $SQLITE_s_DLL)
   If (ErrorLevel <> 0)
   {
      Return ErrorLevel
   }
   Return $SQLITE_OK
}

;===============================================================================
; Function Name:    _SQLite_Shutdown()
; Description:      Unloads SQLite3.dll
; Requirement(s):   Valid Path to SQlite3.dll in $SQLITE_s_DLL
; Parameter(s):     None
; Return Value(s):  On Success - ErrorLevel = 0 = $SQLITE_OK
;                   On Failure - ErrorLevel
;===============================================================================
_SQLite_Shutdown()
{
  Global
   DllCall("FreeLibrary", "UInt", $SQLITE_h_DLL)
   If (ErrorLevel <> 0)
   {
      Return ErrorLevel
   }
   $SQLITE_h_DLL := 0
   Return $SQLITE_OK
}

;===============================================================================
; Function Name:    _SQLite_OpenDB()
; Description:      Opens a Database
;                   DB handle is stored in $SQLITE_h_DB
; Parameter(s):     $sDBFile   - Database Filename
; Return Value(s):  On Success - $SQLITE_OK
;                   On Failure - ErrorLevel/SQLite_RC
;                                Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_OpenDB($sDBFile)
{
   Local $i_RC, $h_DB
   $i_RC := 0
   $h_DB := 0
   
   $SQLITE_s_ERROR := ""
   If $sDBFile =
   {
      $sDBFile := ":memory:"
   }
   $i_RC := DllCall( "sqlite3\sqlite3_open" . (A_IsUnicode ? "16" : "")
                   , "Str", $sDBFile
                   , "Uint *", $h_DB
                   , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall to sqlite3_open failed
      Return Errorlevel
   }
   If ($i_RC <> $SQLITE_OK)
   {
      $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                               , "Uint", $hDB
                               , "Cdecl str")
      Return $i_RC
   }
   $SQLITE_h_DB := $h_DB
   $SQLITE_a_DB%$h_DB% := $h_DB
   Return $i_RC
}

;===============================================================================
; Function Name:    _SQLite_CloseDB()
; Description:      Closes an open Database
;                   Waits until SQLite <> $SQLITE_BUSY until 'Timeout' has elapsed
; Parameter(s):     $hDB       - Database Handle, use -1 for $SQLITE_h_DB
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - ErrorLevel/SQLite_RC
;                                Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_CloseDB($hDB)
{
   Local $i_RC
   $i_RC := 0
   $SQLITE_s_ERROR := ""
   If ($hDB = -1)
   {
      $hDB := $SQLITE_h_DB
   }
   If ($hDB = 0 Or $SQLITE_a_DB%$hDB% = 0)
   {
      $hDB := 0
      Return $SQLITE_OK
   }
   $i_RC := DllCall("sqlite3\sqlite3_close"
                  , "Uint", $hDB
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall failed
      Return Errorlevel
   }
   If ($i_RC <> $SQLITE_OK)
   {
      $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                               , "Uint", $hDB
                               , "Cdecl str")
   }
   If ($hDB = $SQLITE_h_DB)
   {
      $SQLITE_h_DB := 0
   }
   $SQLITE_a_DB%$hDB% := 0
   Return $i_RC
}

;===============================================================================
; Function Name:    _SQlite_Prepare_V2() ; =sqlite3_prepare_V2()  ;After, do sqlite3_finalize() function is called to delete a prepared statement.
; Description:      Prepares a single Statement SQLite Query,
;                   Query Handle is stored in $SQLITE_h_QUERY
; Parameter(s):     $hDB       - Database Handle, use -1 for $SQLITE_h_DB
;                   $sSQL      - SQL Statement to be executed
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - ErrorLevel/SQLite_RC
;                                Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):         MaxMax14
;===============================================================================
_SQlite_Prepare_V2($hDB, $sSQL) 
{ 
   Local $i_RC, $h_Query, $p_SQL
   $i_RC := 0
   $h_Query := 0
   $p_SQL := 0
   $SQLITE_s_ERROR := ""
   If ($hDB = -1)
   {
      $hDB := $SQLITE_h_DB
   }
   If Not _$SQLITE_CheckDB($hDB)
   {
      $SQLITE_s_ERROR = ERROR: DB Handle %$hDB% not valid
      Return $SQLITE_ERROR
   }
   $i_RC := DllCall("sqlite3\sqlite3_prepare" . (A_IsUnicode ? "16" : "") . "_v2"
                  , "Uint", $hDB
                  , "str", $sSQL
                  , "int", -1
                  , "Uint *", $h_Query
                  , "Uint *", $p_SQL
                  , "Cdecl int")

   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall failed
      Return Errorlevel
   }
   If ($i_RC <> $SQLITE_OK)
   {
      $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                               , "Uint", $hDB
                               , "Cdecl str")
   }
   $SQLITE_h_QUERY := $h_QUERY
   $SQLITE_a_QUERY%$h_QUERY% := $h_QUERY
   Return $i_RC
}

;===============================================================================
; Function Name:    _SQLite_Finalize() (Same as _SQLite_QueryFinalize()  ; =sqlite3\sqlite3_finalize >> = _SQLite_QueryFinalize($hQuery) 
; Description:      Finalizes _SQLite_Query() based Query,
;                   Query Handle is not valid any more
; Parameter(s):     $hQuery    - Query Handle, use -1 for $SQLITE_h_QUERY
; Return Value(s):  On Success - $SQLITE_OK
;                   On Failure - ErrorLevel/$SQLITE_ERROR
;                                Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        MaxMax14
;==============================================================================
_SQLite_Finalize($hQuery)
{ 
   Local $i_RC
   $i_RC := 0
   $SQLITE_s_ERROR := ""
   If ($hQuery = -1)
   {
      $hQuery := $SQLITE_h_QUERY
   }
   If Not _$SQLITE_CheckQuery($hQuery)
   {
      $SQLITE_s_ERROR = ERROR: Query Handle %$hQuery% not valid
      Return $SQLITE_ERROR
   }
   $i_RC := DllCall("sqlite3\sqlite3_finalize"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_finalize failed
      Return Errorlevel
   }
   $SQLITE_a_Query%$hQuery% := 0
   If ($hQuery = $SQLITE_h_QUERY)
   {
      $SQLITE_h_QUERY := 0
   }
   If ($i_RC <> $SQLITE_OK)
   {
      $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                               , "Uint", $hDB
                               , "Cdecl str")
   }
   Return $i_RC
}

;===============================================================================
;===============================================================================
;===============================================================================
; Function Name:    _$SQLITE_CheckDB()
; Description:      Validates the given Database Handle
; Parameter(s):     $hDB       - Database Handle
; Return Value(s):  On Success - Returns True
;                   On Failure - Returns False
; Author(s):        nick
;===============================================================================
_$SQLITE_CheckDB($hDB)
{
   Global
   If $hDB Is Not Integer
   {
      Return False
   }
   If ($hDB = 0)
   {
      Return False
   }
   If ($SQLITE_a_DB%$hDB% <> $hDB)
   {
      Return False
   }
   Return True
}

;===============================================================================
; Function Name:    _$SQLITE_CheckQuery()
; Description:      Validates the given Query Handle
; Parameter(s):     $hQuery    - Query Handle
; Return Value(s):  On Success - Returns True
;                   On Failure - Returns False
; Author(s):        nick
;===============================================================================
_$SQLITE_CheckQuery($hQuery)
{
   Global
   If $hQuery Is Not Integer
   {
      Return False
   }
   If ($hQuery = 0)
   {
      Return False
   }
   If ($SQLITE_a_Query%$hQuery% <> $hQuery)
   {
      Return False
   }
   Return True
}

;===============================================================================
; Function Name:    _SQLite_Exec()
;
; Description:      Executes a non Query SQLite Statement,
;                   Does not handle Results
; Parameter(s):     $hDB       - Open database handle, use -1 for $SQLITE_h_DB
;                   $sSQL      - SQL Statement to be executed
; Return Value(s):  On Success - Returns $SQLITE_OK
;                   On Failure - ErrorLevel/SQLite_RC
;                                Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_Exec($hDB, $sSQL)
{
   Local $i_RC, $iErr 
   $i_RC := 0
   $SQLITE_s_ERROR := ""
   If ($hDB = -1)
   {
      $hDB := $SQLITE_h_DB
   }
   If Not _$SQLITE_CheckDB($hDB)
   {
      $SQLITE_s_ERROR = ERROR: DB Handle %$hDB% not valid
      Return $SQLITE_ERROR
   }
   ; Prepare_V2 
   $i_RC := _SQlite_Prepare_V2($hDB, $sSQL)
   If ($i_RC <> $SQLITE_OK)
   {
     Return $i_RC
   }
   $hQuery := $SQLITE_h_QUERY
   
   ; Step
   $i_RC := DllCall("sqlite3\sqlite3_step"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_step failed
      Return Errorlevel
   }
   If ($i_RC <> $SQLITE_DONE)
   {
      $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                              , "Uint", $hDB
                              , "Cdecl str")
   }
   Else
   $i_RC := $SQLITE_OK

   ; Finalize
   _SQLite_Finalize($hQuery) 
   Return $i_RC
}

;===============================================================================
; Function Name:    _SQLite_FetchNames()  ;=sqlite3\sqlite3_column_count + sqlite3\sqlite3_column_name"
; Description:      Read out the Column Names of a _SQLite_Query() based Query
; Parameter(s):     $hQuery     - Query Handle, use -1 for $SQLITE_h_QUERY
;                   ByRef $sNames - Passes out Column Names
; Return Value(s):  On Success    - $SQLITE_OK
;                   On Failure    - ErrorLevel/SQLite_RC
;                                   Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_FetchNames($hQuery, ByRef $sNames)
{
   Local $i_RC, $i_Col, $s_Col
   $i_RC := 0
   $i_Col := 0
   $s_Col := ""
   $sNames := ""
   $SQLITE_s_ERROR := ""
   If ($hQuery = -1)
   {
      $hQuery := $SQLITE_h_QUERY
   }
   If Not _$SQLITE_CheckQuery($hQuery)
   {
      $SQLITE_s_ERROR = ERROR: Query Handle %$hQuery% not valid
      Return $SQLITE_ERROR
   }
   $i_RC := DllCall("sqlite3\sqlite3_column_count"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_column_count failed
      Return Errorlevel
   }
   If ($i_RC < 1)
   {
      $SQLITE_s_ERROR = ERROR: Query result is empty
      Return $SQLITE_EMPTY
   }
   Loop %$i_RC%
   {
      $s_Col := DllCall("sqlite3\sqlite3_column_name" . (A_IsUnicode ? "16" : "")
                      , "Uint", $hQuery
                      , "int", $i_Col
                      , "Cdecl str")
      If (ErrorLevel <> 0)
      {
         $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_column_name failed
         Return Errorlevel
      }
      $sNames := $sNames . $s_Col . "|"
      $i_Col += 1
   }
   If ($sNames <> "")
   {
      StringTrimRight $sNames, $sNames, 1
   }
   Return $SQLITE_OK
}

;===============================================================================
; Function Name:    _SQLite_FetchData() ;=sqlite3\sqlite3_step + sqlite3\sqlite3_data_count + sqlite3\sqlite3_column_type + sqlite3\sqlite3_column_text
; Description:      Fetches 1 Row of Data from a _SQLite_Query() based Query
; Parameter(s):     $hQuery     - Query Handle, use -1 for $SQLITE_h_QUERY
;                   ByRef $sRow - Passes out a Row of Data
; Return Value(s):  On Success  - $SQLITE_OK
;                   On Failure  - ErrorLevel/SQLite_RC
;                                 Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_FetchData($hQuery, ByRef $sRow)
{
   Local $i_RC, $i_Col, $i_Type, $s_Col, $b_Done
   Local $SQLITE_NULL
   $SQLITE_NULL = 5
   $i_RC := 0
   $i_Col := 0
   $b_Done := False
   $sRow := ""
   $SQLITE_s_ERROR := ""
   If ($hQuery = -1)
   {
      $hQuery := $SQLITE_h_QUERY
   }
   If Not _$SQLITE_CheckQuery($hQuery)
   {
      $SQLITE_s_ERROR = ERROR: Query Handle %$hQuery% not valid
      Return $SQLITE_ERROR
   }
   $i_RC := DllCall("sqlite3\sqlite3_step"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_step failed
      Return Errorlevel
   }
   If ($i_RC <> $SQLITE_ROW)
   {
      If ($i_RC <> $SQLITE_DONE)
      {
         $SQLITE_s_ERROR := DllCall("sqlite3\sqlite3_errmsg" . (A_IsUnicode ? "16" : "")
                                  , "Uint", $hDB
                                  , "Cdecl str")
      }
      _SQLite_Finalize($hQuery)
      Return $i_RC
   }
   $i_RC := DllCall("sqlite3\sqlite3_data_count"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_data_count failed
      Return Errorlevel
   }
   If ($i_RC < 1)
   {
      $SQLITE_s_ERROR = ERROR: Query result is empty
      Return $SQLITE_EMPTY
   }
   Loop %$i_RC%
   {
      $i_Type := DllCall("sqlite3\sqlite3_column_type"
                       , "Uint", $hQuery
                       , "int", $i_Col
                       , "Cdecl int")
      If (ErrorLevel <> 0)
      {
         $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_column_type failed
         Return Errorlevel
      }
      If ($i_Type = $SQLITE_NULL)
      {
         $s_Col := ""
      }
      Else
      {
      $s_Col := DllCall("sqlite3\sqlite3_column_text" . (A_IsUnicode ? "16" : "")
                      , "Uint", $hQuery
                      , "int", $i_Col
                      , "Cdecl str")
         If (ErrorLevel <> 0)
         {
            $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_column_text failed
            Return Errorlevel
         }
      }
      $sRow := $sRow . $s_Col . "|"
      $i_Col += 1
   }
   If ($sRow <> "")
   {
      StringTrimRight $sRow, $sRow, 1
   }
   Return $SQLITE_OK
}

;===============================================================================
; Function Name:    _SQLite_GetTable()  ; =sqlite3\sqlite3_get_table + SQLite3\sqlite3_free_table
; http://www.sqlite.org/c3ref/free_table.html
;
; Description:      Passes out a String containing Column Names and Data of a
;                   given Query. Columns are seperated by "|", Rows by "`n"
; Parameter(s):     $hDB           - Database Handle, use -1 for $SQLITE_h_DB
;                   $sSQL          - SQL Statement to be executed
;                   ByRef $sResult - Passes out the Resultstring
;                   ByRef $iRows   - Passes out the Amount of 'data' Rows
;                   ByRef $iCols   - Passes out the Amount of Columns
;                   $iMaxResult    - Optional: Amount of Rows to be returned
;                                    Default = -1 : All Rows
;                                    Specify 0 to get only $iRows and $iCols
;                                    Specify 1 to get Column Names also 
;                   $iCharSize     - Optional: Maximal Size of a Data Field
; Return Value(s):  On Success     - Returns $SQLITE_OK
;                   On Failure     - ErrorLevel/SQLite_RC
;                                    Errormessage is stored in $SQLITE_s_ERROR 
; Author(s):        nick
;===============================================================================
_SQLite_GetTable($hDB, $sSQL, ByRef $sResult, ByRef $iRows, ByRef $iCols
                 , $iMaxResult = -1, $iCharSize = 64)
{ 
   Local $i_RC, $iErr , $sTemp
   
   $sResult=
   $iRows=0
   $iCols=0
   
   $i_RC := 0
   $SQLITE_s_ERROR := ""
   If ($hDB = -1)
   {
      $hDB := $SQLITE_h_DB
   }

   If Not _$SQLITE_CheckDB($hDB)
   {
      $SQLITE_s_ERROR = ERROR: DB Handle %$hDB% not valid
      $sResult=
      Return $SQLITE_ERROR
   }

   ; Prepare_V2 
   $i_RC := _SQlite_Prepare_V2($hDB, $sSQL)
   If ($i_RC <> $SQLITE_OK)
   {
     $sResult=
     Return $i_RC
   }
   $hQuery := $SQLITE_h_QUERY

   ; Get Column Count
   $iCols := DllCall("sqlite3\sqlite3_column_count"
                  , "Uint", $hQuery
                  , "Cdecl int")
   If (ErrorLevel <> 0)
   {
      $SQLITE_s_ERROR = ERROR: DLLCall sqlite3_column_count failed
      $sResult=
      Return Errorlevel
   }
   If ($iCols < 1)
   {
      $SQLITE_s_ERROR = ERROR: Query result is empty
      Return $SQLITE_EMPTY
   }

   ; Get Column Name
   $i_RC := _SQLite_FetchNames($hQuery, $sTemp)
   If ($i_RC <> $SQLITE_OK)
   {
     $sResult=
     Return $i_RC
   }
   $sResult := $sResult . $sTemp . "`n"

   ; Get Data
   $iRows=0
   Loop
   {
      $i_RC := _SQLite_FetchData($hQuery, $sTemp)
      If ($i_RC <> $SQLITE_OK)
         break

      $sResult := $sResult . $sTemp . "`n"
      $iRows ++
   }

   $sResult:=RegExReplace($sResult, "\n+$" ,"")

   ; Finalize
   _SQLite_Finalize($hQuery) 
   $SQLITE_s_ERROR := ""
   $i_RC := $SQLITE_OK
   Return $i_RC
}
